@@ -13,8 +13,10 @@ from typing import Iterable, List, Optional, TypeVar, Type, Tuple, Dict, Union
 import imaplib
 
 from googleapiclient.discovery import build, Resource
-from google.oauth2 import service_account
+from googleapiclient.errors import HttpError
+from google.auth.exceptions import RefreshError
 from google.auth.transport.requests import Request
+from google.oauth2 import service_account
 from google.oauth2.credentials import Credentials
 
 from django.conf import settings
@@ -25,6 +27,7 @@ from pydantic.error_wrappers import ValidationError  # pylint: disable=no-name-i
 from nautobot.circuits.models import Provider
 from nautobot.extras.jobs import Job
 
+from nautobot_circuit_maintenance.enum import MessageProcessingStatus
 from nautobot_circuit_maintenance.models import NotificationSource
 
 logger = logging.getLogger(__name__)
@@ -35,17 +38,6 @@ logger = logging.getLogger(__name__)
 T = TypeVar("T", bound="Source")  # pylint: disable=invalid-name
 
 
-class MaintenanceNotification(BaseModel):
-    """Representation of all the data related to a Maintenance Notification."""
-
-    source: str
-    sender: str
-    subject: str
-    provider_type: str
-    raw_payload: bytes
-    date: str
-
-
 class Source(BaseModel):
     """Base class to retrieve notifications. To be extended for each scheme."""
 
@@ -58,7 +50,7 @@ class Source(BaseModel):
 
     def receive_notifications(
         self, job_logger: Job, since_timestamp: datetime.datetime = None
-    ) -> Iterable[MaintenanceNotification]:
+    ) -> Iterable["MaintenanceNotification"]:
         """Function to retrieve notifications since one moment in time.
 
         The `MaintenanceNotification` attributes will contains these attributes:
@@ -164,12 +156,31 @@ class Source(BaseModel):
                     source_header=config.get("source_header", "From"),
                     limit_emails_with_not_header_from=config.get("limit_emails_with_not_header_from", []),
                     extra_scopes=config.get("extra_scopes", []),
+                    labels=config.get("labels", {}),
                 )
 
         raise ValueError(
             f"Scheme {scheme} not supported as Notification Source (only IMAP or HTTPS to accounts.google.com)."
         )
 
+    def tag_message(self, job_logger: Job, msg_id: Union[str, bytes], tag: MessageProcessingStatus):
+        """If supported, apply the given tag to the given message for future reference and categorization.
+
+        The default implementation of this method is a no-op but specific Source subclasses may implement it.
+        """
+
+
+class MaintenanceNotification(BaseModel):
+    """Representation of all the data related to a Maintenance Notification."""
+
+    msg_id: bytes
+    source: Source
+    sender: str
+    subject: str
+    provider_type: str
+    raw_payload: bytes
+    date: str
+
 
 class EmailSource(Source):  # pylint: disable=abstract-method
     """Abstract class that shares some methods and attributes accross email based sources."""
@@ -260,9 +271,9 @@ class EmailSource(Source):  # pylint: disable=abstract-method
         return None
 
     def process_email(
-        self, job_logger: Job, email_message: email.message.EmailMessage
+        self, job_logger: Job, email_message: email.message.EmailMessage, msg_id: Union[str, bytes]
     ) -> Optional[MaintenanceNotification]:
-        """Process an EmailMessage to create the MaintenaceNotification."""
+        """Process an EmailMessage to create the MaintenanceNotification."""
         email_source = None
         if email_message[self.source_header]:
             email_source = self.extract_email_source(email_message[self.source_header])
@@ -273,20 +284,23 @@ class EmailSource(Source):  # pylint: disable=abstract-method
                 message="Not possible to determine the email sender from "
                 f'"{self.source_header}: {email_message[self.source_header]}"',
             )
+            self.tag_message(job_logger, msg_id, MessageProcessingStatus.UNKNOWN_PROVIDER)
             return None
 
         provider_type = self.get_provider_type_from_email(email_source)
         if not provider_type:
             job_logger.log_warning(message=f"Not possible to determine the provider_type for {email_source}")
+            self.tag_message(job_logger, msg_id, MessageProcessingStatus.UNKNOWN_PROVIDER)
             return None
 
         return MaintenanceNotification(
-            source=self.name,
+            source=self,
             sender=email_source,
             subject=email_message["Subject"],
             provider_type=provider_type,
             raw_payload=email_message.as_bytes(),
             date=email_message["Date"],
+            msg_id=msg_id,
         )
 
 
@@ -333,10 +347,9 @@ class IMAP(EmailSource):
     def fetch_email(self, job_logger: Job, msg_id: bytes) -> Optional[MaintenanceNotification]:
         """Fetch an specific email ID."""
         _, data = self.session.fetch(msg_id, "(RFC822)")
-        raw_email_string = data[0][1].decode("utf-8")
-        email_message = email.message_from_string(raw_email_string)
+        email_message = email.message_from_bytes(data[0][1])
 
-        return self.process_email(job_logger, email_message)
+        return self.process_email(job_logger, email_message, msg_id)
 
     def receive_notifications(
         self, job_logger: Job, since_timestamp: datetime.datetime = None
@@ -407,17 +420,19 @@ class GmailAPI(EmailSource):
     service: Optional[Resource] = None
     credentials: Optional[Union[service_account.Credentials, Credentials]] = None
 
+    # The required scope for baseline functionality (add gmail.modify permission in extra_scopes to enable tagging)
     SCOPES = ["https://www.googleapis.com/auth/gmail.readonly"]
 
     extra_scopes: List[str] = []
     limit_emails_with_not_header_from: List[str] = []
+    labels: Dict[str, str] = {}
 
     class Config:
         """Pydantic BaseModel config."""
 
         arbitrary_types_allowed = True
 
-    def load_credentials(self):
+    def load_credentials(self, force_refresh=False):
         """Load Credentials for Gmail API."""
         raise NotImplementedError
 
@@ -432,9 +447,9 @@ class GmailAPI(EmailSource):
 
     def _authentication_logic(self):
         """Inner method to run the custom class validation logic."""
-        self.load_credentials()
+        self.load_credentials(force_refresh=True)
 
-    def extract_raw_payload(self, body: Dict, msg_id: bytes) -> bytes:
+    def extract_raw_payload(self, body: Dict, msg_id: str) -> bytes:
         """Extracts the raw_payload from body or attachement."""
         if "attachmentId" in body:
             attachment = (
@@ -450,7 +465,7 @@ class GmailAPI(EmailSource):
 
         return b""
 
-    def fetch_email(self, job_logger: Job, msg_id: bytes) -> Optional[MaintenanceNotification]:
+    def fetch_email(self, job_logger: Job, msg_id: str) -> Optional[MaintenanceNotification]:
         """Fetch an specific email ID.
 
         See data format:  https://developers.google.com/gmail/api/reference/rest/v1/users.messages#Message
@@ -464,7 +479,7 @@ class GmailAPI(EmailSource):
 
         raw_email_string = base64.urlsafe_b64decode(received_email["raw"].encode("utf8"))
         email_message = email.message_from_bytes(raw_email_string)
-        return self.process_email(job_logger, email_message)
+        return self.process_email(job_logger, email_message, msg_id)
 
     def _get_search_criteria(self, since_timestamp: datetime.datetime = None) -> str:
         """Build "search" criteria to filter emails, from date of from sender."""
@@ -474,7 +489,7 @@ class GmailAPI(EmailSource):
             search_criteria = f"after:{since_txt}"
 
         # If source_header is not "From" but some other custom header such as X-Original-Sender,
-        # the GMail API doesn't let us filter by that, but if we provided via config a list of
+        # the Gmail API doesn't let us filter by that, but if we provided via config a list of
         # source via `limit_emails_with_not_header_from`, we filter by that.
         if self.emails_to_fetch and self.source_header == "From":
             emails_with_from = [f"from:{email}" for email in self.emails_to_fetch]
@@ -485,6 +500,28 @@ class GmailAPI(EmailSource):
 
         return search_criteria
 
+    def tag_message(self, job_logger: Job, msg_id: Union[str, bytes], tag: MessageProcessingStatus):
+        """Apply the given Gmail label to the given message."""
+        # Do we have a configured label ID corresponding to the given tag?
+        if tag.value not in self.labels:
+            return
+
+        # Gmail API expects a str msg_id, but MaintenanceNotification coerces it to bytes - change it back if needed
+        if isinstance(msg_id, bytes):
+            msg_id = str(msg_id.decode())
+
+        try:
+            self.service.users().messages().modify(  # pylint: disable=no-member
+                userId=self.account, id=msg_id, body={"addLabelIds": [self.labels[tag.value]]}
+            ).execute()
+        except HttpError as exc:
+            job_logger.log_warning(
+                message=(
+                    f"Error in applying tag '{tag.value}' ({self.labels[tag.value]}) to message {msg_id}: "
+                    f"{exc.status_code} {exc.error_details}"
+                ),
+            )
+
     def receive_notifications(
         self, job_logger: Job, since_timestamp: datetime.datetime = None
     ) -> Iterable[MaintenanceNotification]:
@@ -544,7 +581,7 @@ class GmailAPIOauth(GmailAPI):
     See: https://developers.google.com/identity/protocols/oauth2/web-server
     """
 
-    def load_credentials(self):
+    def load_credentials(self, force_refresh=False):
         """Load Gmail API OAuth credentials."""
         notification_source = NotificationSource.objects.get(name=self.name)
         try:
@@ -552,9 +589,16 @@ class GmailAPIOauth(GmailAPI):
         except EOFError:
             logger.debug("Google OAuth Token has not been initialized yet.")
 
-        if not self.credentials or not self.credentials.valid:
-            if self.credentials and self.credentials.expired and self.credentials.refresh_token:
-                self.credentials.refresh(Request())
+        if force_refresh or not self.credentials or not self.credentials.valid:
+            if self.credentials and self.credentials.refresh_token and (self.credentials.expired or force_refresh):
+                try:
+                    self.credentials.refresh(Request())
+                except RefreshError:
+                    # Bad token, discard it
+                    notification_source._token = b""  # pylint: disable=protected-access
+                    notification_source.save()
+                    raise
+
                 notification_source.token = self.credentials
                 notification_source.save()
             else:
@@ -567,9 +611,9 @@ class GmailAPIOauth(GmailAPI):
 class GmailAPIServiceAccount(GmailAPI):
     """GmailAPIServiceAccount class."""
 
-    def load_credentials(self):
+    def load_credentials(self, force_refresh=False):
         """Load Gmail API Service Account credentials."""
-        if not self.credentials:
+        if force_refresh or not self.credentials:
             self.credentials = service_account.Credentials.from_service_account_file(self.credentials_file)
             self.credentials = self.credentials.with_scopes(self.SCOPES + self.extra_scopes)
             self.credentials = self.credentials.with_subject(self.account)
